import { remove } from 'lodash';
import { loadASMAMicroAPP } from '../loadASMAMicroApp';
const LoaderQueue = {};
const occurences = {};
function incrementOccurence(app_name) {
    if (!occurences[app_name]) {
        occurences[app_name] = 0;
    }
    occurences[app_name]++;
}
function registerLoader(app_name, loader) {
    if (!LoaderQueue[app_name]) {
        LoaderQueue[app_name] = [];
    }
    LoaderQueue[app_name].push(loader);
}
const areLoadersInProcess = {};
async function resolveLoaders(app_name) {
    var _a, _b;
    areLoadersInProcess[app_name] = true;
    while (areLoadersInProcess[app_name]) {
        const loader_to_resolve = (_a = LoaderQueue[app_name]) === null || _a === void 0 ? void 0 : _a[0];
        if (loader_to_resolve) {
            await loader_to_resolve.init().bootstrapPromise;
            remove(LoaderQueue[app_name] || [], (l) => l.id === loader_to_resolve.id);
        }
        if (!((_b = LoaderQueue[app_name]) === null || _b === void 0 ? void 0 : _b.length)) {
            areLoadersInProcess[app_name] = false;
        }
    }
}
let initLoadMicroApp;
function initLoadMicroAppFn(app, props, containerRef, setLoadedApp) {
    function init() {
        const loadedapp = loadASMAMicroAPP({
            name: app.name,
            entry: app.entry,
            container: containerRef.current,
            props: { ...props, occurence: occurences[app.name] },
        });
        setLoadedApp(loadedapp);
        return loadedapp;
    }
    incrementOccurence(app.name);
    registerLoader(app.name, { id: props.component_path, init });
    if (!areLoadersInProcess[app.name]) {
        resolveLoaders(app.name);
    }
}
initLoadMicroApp = window.__INIT_LOAD_MICROAPP__ || initLoadMicroAppFn;
if (!window.__INIT_LOAD_MICROAPP__) {
    window.__INIT_LOAD_MICROAPP__ = initLoadMicroApp;
}
export { initLoadMicroApp };
//# sourceMappingURL=LoaderQueue.js.map